import os

import pytest

import valohai
import valohai.paths


@pytest.mark.parametrize("name", ("", "myoutput"))
@pytest.mark.parametrize("fragment", ("jahas.exe", "herpderp/yomomma/joo.txt"))
def test_get_output_path(outputs_path, name, fragment):
    path = valohai.outputs(name).path(fragment)
    assert path == os.path.join(outputs_path, name, fragment)
    assert os.path.isdir(os.path.dirname(path))


def test_live_upload(outputs_path):
    path = valohai.outputs().path("hello.txt")
    with open(path, "w") as fp:
        fp.write("hello")
    valohai.outputs().live_upload(path)

    with pytest.raises(IOError):
        # Test the file is set read-only
        open(path, "w")



import glob
import json
import os


def read_source_files(path_without_ext):
    with open(f"{path_without_ext}.py") as source_python:
        source = source_python.read()
    with open(f"{path_without_ext}.parameters.json") as parameters_json:
        parameters = json.load(parameters_json)
    with open(f"{path_without_ext}.inputs.json") as inputs_json:
        inputs = json.load(inputs_json)
    with open(f"{path_without_ext}.step.json") as step_json:
        step = json.load(step_json)
    return {
        "source": source,
        "parameters": parameters,
        "inputs": inputs,
        "step": step,
    }


def get_parsing_tests():
    basepath = os.path.join(os.path.dirname(os.path.abspath(__file__)), "test_parsing")

    for python_file in glob.glob(os.path.join(basepath, "*.py")):
        path_without_ext = os.path.splitext(python_file)[0]
        yield read_source_files(path_without_ext)



import os

import pytest

from .valohai_test_environment import ValohaiTestEnvironment


@pytest.fixture
def vte(tmpdir):
    return ValohaiTestEnvironment(root_dir=str(tmpdir.mkdir("valohai")))


@pytest.fixture
def use_test_config_dir(vte, monkeypatch):
    vte.build()
    monkeypatch.setenv("VH_CONFIG_DIR", str(vte.config_path))
    monkeypatch.setenv("VH_INPUT_DIR", str(vte.inputs_path))
    monkeypatch.setenv("VH_OUTPUT_DIR", str(vte.outputs_path))


@pytest.fixture
def outputs_path(tmpdir, monkeypatch):
    outputs_path = os.path.join(str(tmpdir), "outputs")
    monkeypatch.setenv("VH_OUTPUTS_DIR", outputs_path)
    return outputs_path


@pytest.fixture
def output_files(outputs_path):
    os.makedirs(os.path.join(outputs_path, "folder"))
    os.makedirs(os.path.join(outputs_path, "folder2"))

    outputs = [
        os.path.join(outputs_path, "test1.bin"),
        os.path.join(outputs_path, "test2.bin"),
        os.path.join(outputs_path, "folder", "picture.jpg"),
        os.path.join(outputs_path, "folder", "picturetoo.jpg"),
        os.path.join(outputs_path, "folder", "imapng.png"),
        os.path.join(outputs_path, "folder2", "asdf.dat"),
    ]

    for output in outputs:
        with open(output, "wb") as f:
            f.write(os.urandom(1000))

    return outputs



import os
import sys

import valohai
from valohai.internals.download_type import DownloadType
from valohai.internals.input_info import InputInfo, load_input_info


def test_prepare(tmpdir, monkeypatch):
    inputs_dir = str(tmpdir.mkdir("inputs"))
    monkeypatch.setenv("VH_INPUTS_DIR", inputs_dir)
    local_file = tmpdir.mkdir("sub").join("hello.txt")
    local_file.write("tiku ja taku ja joku")

    parameters = {
        "iambool": True,
        "mestringy": "asdf",
        "integerboi": 123,
        "floaty": 0.0001,
        "makemetrue": False,
        "makemeqwer": "asdf",
        "makeme321": 123,
        "makemenegative": 0.0001,
    }
    inputs = {
        "example": "https://valohai-mnist.s3.amazonaws.com/t10k-images-idx3-ubyte.gz",
        "overrideme": "https://valohai-mnist.s3.amazonaws.com/t10k-images-idx3-ubyte.gz",
        "myimages": [
            "https://upload.wikimedia.org/wikipedia/commons/8/84/Example.svg",
            "https://upload.wikimedia.org/wikipedia/commons/0/01/Example_Wikipedia_sandbox_move_UI.png",
        ],
    }

    with monkeypatch.context() as m:
        args = [
            "",
            "--makemetrue=true",
            "--makemeqwer=qwer",
            "--makeme321=321",
            "--makemenegative=-0.123",
            "--some_totally_random_parameter_to_ignore=666",
            f"--overrideme={str(local_file)}",
        ]
        m.setattr(
            sys,
            "argv",
            args,
        )
        valohai.prepare(
            step="test", default_parameters=parameters, default_inputs=inputs
        )

    assert valohai.parameters("iambool").value == True
    assert valohai.parameters("mestringy").value == "asdf"
    assert valohai.parameters("integerboi").value == 123
    assert valohai.parameters("floaty").value == 0.0001
    assert valohai.parameters("makemetrue").value == True
    assert valohai.parameters("makemeqwer").value == "qwer"
    assert valohai.parameters("makeme321").value == 321
    assert valohai.parameters("makemenegative").value < 0.0

    assert (
        load_input_info("example", download=DownloadType.NEVER).files[0].uri
        == "https://valohai-mnist.s3.amazonaws.com/t10k-images-idx3-ubyte.gz"
    )
    assert (
        load_input_info("myimages", download=DownloadType.NEVER).files[0].uri
        == "https://upload.wikimedia.org/wikipedia/commons/8/84/Example.svg"
    )
    assert (
        load_input_info("myimages", download=DownloadType.NEVER).files[1].uri
        == "https://upload.wikimedia.org/wikipedia/commons/0/01/Example_Wikipedia_sandbox_move_UI.png"
    )
    assert not load_input_info("overrideme", download=DownloadType.NEVER).files[0].uri
    assert os.path.isfile(
        load_input_info("overrideme", download=DownloadType.NEVER).files[0].path
    )



import glob
import os
import shutil

import pytest
from valohai_yaml import parse

from valohai.internals.merge import python_to_yaml_merge_strategy
from valohai.internals.yaml import parse_config_from_source
from valohai.yaml import config_to_yaml


def read_test_data():
    """
    Expected files (tests/test_yaml):
        mytest.py -- Python file calling valohai.prepare()
        mytest.original.valohai.yaml -- Original valohai.yaml
        mytest.expected.valohai.yaml -- Expected valohai.yaml after update
    """
    test_data = []
    for source_path in glob.glob("tests/test_yaml/*.py"):
        dirname = os.path.dirname(source_path)
        name, extension = os.path.splitext(os.path.basename(source_path))
        test_data.append(
            (
                f"{dirname}/{name}.original.valohai.yaml",
                source_path,
                f"{dirname}/{name}.expected.valohai.yaml",
            )
        )
    return test_data


@pytest.mark.parametrize(
    "original_yaml, source_python, expected_yaml", read_test_data()
)
def test_yaml_update_from_source(tmpdir, original_yaml, source_python, expected_yaml):
    yaml_path = os.path.join(tmpdir, "valohai.yaml")
    source_path = os.path.join(tmpdir, "test.py")

    # Build repository with test.py and valohai.yaml
    if os.path.isfile(original_yaml):
        shutil.copy(original_yaml, yaml_path)
    shutil.copy(source_python, source_path)

    # Load original valohai.yaml
    old_config = None
    if os.path.isfile(yaml_path):
        with open(yaml_path) as yaml_file:
            old_config = parse(yaml_file)

    # Parse new config from .py
    new_config = parse_config_from_source(source_path, yaml_path)

    # Merge original and new
    if old_config:
        new_config = old_config.merge_with(new_config, python_to_yaml_merge_strategy)

    # Check against expected result
    with open(expected_yaml) as expected_yaml:
        new_yaml = config_to_yaml(new_config)
        assert new_yaml == expected_yaml.read()



import io
import os
import tarfile
import zipfile

from valohai.internals.vfs import find_files, VFS


def _add_binary_to_tar(tf: tarfile.TarFile, name: str, content: bytes):
    tarinfo = tarfile.TarInfo(name)
    tarinfo.size = len(content)
    tf.addfile(tarinfo, io.BytesIO(content))


expected_contents = {
    "0quux.txt": b"Maista uudet maut",
    "dir1/1hello.txt": b"Hernekeitto",
    "dir1/2world.txt": b"Viina",
    "dir2/3spam.txt": b"Teline",
    "dir2/4eggs.txt": b"Johannes",
}


def make_vfs_files(tmpdir):
    zip_path = tmpdir.join("dir1").mkdir().join("example.zip")
    tar_path = tmpdir.join("dir2").mkdir().join("esimerkki.tar")
    raw_path = tmpdir.join("0quux.txt")

    with zipfile.ZipFile(zip_path, "w") as zf:
        zf.writestr("1hello.txt", b"Hernekeitto")
        zf.writestr("2world.txt", b"Viina")

    with tarfile.TarFile.open(tar_path, "w") as tf:
        _add_binary_to_tar(tf, "3spam.txt", b"Teline")
        _add_binary_to_tar(tf, "4eggs.txt", b"Johannes")

    with open(raw_path, "w") as outf:
        outf.write("Maista uudet maut")


def test_vfs(tmpdir):
    make_vfs_files(tmpdir)
    names_that_should_be_cleaned = set()
    with VFS() as vfs:
        find_files(vfs, tmpdir, process_archives=True)
        direct_contents = {}
        open_read_contents = {}
        concrete_contents = {}
        for file in vfs.files:
            direct_contents[file.name] = file.read()

            with file.open() as of:
                open_read_contents[file.name] = of.read()

            with (file.open() if not file.parent_file else file.open_concrete()) as cf:
                # If os.open() can read the file, it's a concrete disk file
                fd = os.open(cf.name, os.R_OK)
                concrete_contents[file.name] = os.read(fd, 10_000)
                os.close(fd)
                if file.parent_file:
                    names_that_should_be_cleaned.add(cf.name)
        assert direct_contents == expected_contents
        assert concrete_contents == expected_contents
        assert open_read_contents == expected_contents

    assert not any(os.path.isfile(name) for name in names_that_should_be_cleaned)



import valohai


def test_basic_logging(capsys):
    with valohai.logger() as logger:
        logger.log("myint", 123)
    captured = capsys.readouterr()
    assert captured.out == '\n{"myint": 123}\n'

    with valohai.logger() as logger:
        logger.log("I am float", 0.1241234435877)
    captured = capsys.readouterr()
    assert captured.out == '\n{"I am float": 0.1241234435877}\n'


def test_partial_logging(capsys):
    with valohai.logger() as logger:
        logger.log("myint", 123)
        logger.log("myfloat", 0.5435)
    captured = capsys.readouterr()
    assert captured.out == '\n{"myint": 123, "myfloat": 0.5435}\n'

    logger = valohai.logger()
    logger.log("myint", 234)
    logger.log("myfloat", 0.2322323)
    logger.log("extrathing", 2)
    logger.flush()
    captured = capsys.readouterr()
    assert captured.out == '\n{"myint": 234, "myfloat": 0.2322323, "extrathing": 2}\n'



import os

import valohai


def test_get_input_paths(use_test_config_dir):
    assert valohai.inputs("single_image").path().endswith("single_image/Example.jpg")
    assert os.path.exists(valohai.inputs("single_image").path())
    assert (
        valohai.inputs("single_image")
        .path(default="unused_default")
        .endswith("single_image/Example.jpg")
    )
    assert not valohai.inputs("nonono").path()
    assert valohai.inputs("nonono").path(default="default_123") == "default_123"
    assert os.path.exists(valohai.inputs("input_with_archive").path())
    for path in valohai.inputs("input_with_archive").paths():
        assert os.path.exists(path)
    assert len(list(valohai.inputs("input_with_archive").paths())) == 2


def test_get_input_streams(use_test_config_dir):
    assert valohai.inputs("single_image").stream().read(10000)
    assert len(list(valohai.inputs("input_with_archive").streams())) == 2
    for stream in valohai.inputs("input_with_archive").streams():
        assert stream.read(10000)
    assert valohai.inputs("single_image").stream().read()
    assert not valohai.inputs("nonono").stream()



import os
import sys

import valohai
from valohai.internals.input_info import InputInfo, load_input_info


def test_download(tmpdir, monkeypatch, requests_mock):
    inputs_dir = str(tmpdir.mkdir("inputs"))
    monkeypatch.setenv("VH_INPUTS_DIR", inputs_dir)

    requests_mock.get(
        "https://valohai-mnist.s3.amazonaws.com/t10k-images-idx3-ubyte.gz"
    )
    requests_mock.get("https://upload.wikimedia.org/wikipedia/commons/8/84/Example.svg")
    requests_mock.get(
        "https://upload.wikimedia.org/wikipedia/commons/0/01/Example_Wikipedia_sandbox_move_UI.png"
    )

    inputs = {
        "example": "https://valohai-mnist.s3.amazonaws.com/t10k-images-idx3-ubyte.gz",
        "myimages": [
            "https://upload.wikimedia.org/wikipedia/commons/8/84/Example.svg",
            "https://upload.wikimedia.org/wikipedia/commons/0/01/Example_Wikipedia_sandbox_move_UI.png",
        ],
    }

    monkeypatch.setattr(sys, "argv", ["myscript.py"])
    valohai.prepare(step="test", default_inputs=inputs)

    # These calls will trigger downloads
    assert (
        load_input_info("example").files[0].uri
        == "https://valohai-mnist.s3.amazonaws.com/t10k-images-idx3-ubyte.gz"
    )
    assert (
        load_input_info("myimages").files[0].uri
        == "https://upload.wikimedia.org/wikipedia/commons/8/84/Example.svg"
    )
    assert (
        load_input_info("myimages").files[1].uri
        == "https://upload.wikimedia.org/wikipedia/commons/0/01/Example_Wikipedia_sandbox_move_UI.png"
    )

    assert requests_mock.call_count == 3

    assert os.path.isfile(
        os.path.join(inputs_dir, "example", "t10k-images-idx3-ubyte.gz")
    )
    assert os.path.isfile(os.path.join(inputs_dir, "myimages", "Example.svg"))
    assert os.path.isfile(
        os.path.join(inputs_dir, "myimages", "Example_Wikipedia_sandbox_move_UI.png")
    )

    # Second time around, the file should be cached and not trigger any more downloads
    load_input_info("myimages")
    assert requests_mock.call_count == 3



from typing import IO, Iterable, Iterator, Optional

from valohai.internals.download_type import DownloadType

from .internals import vfs
from .internals.input_info import load_input_info


class Input:
    def __init__(self, name: str):
        self.name = str(name)

    def paths(
        self,
        default: Optional[Iterable[str]] = None,
        process_archives: bool = True,
        force_download: bool = False,
    ) -> Optional[Iterable[str]]:
        """Get paths to all files for a given input name.

        Returns a list of file system paths for an input.
        If the input is not found in the cache, it is downloaded automatically.

        See streams() or path() for alternatives.

        :param default: Default fallback paths.
        :param process_archives: When facing an archive file, is it unpacked to several paths or returned as is
        :param force_download: Force re-download of file(s) even when they are cached already.
        :return: List of file system paths for all the files for this input.
        """
        if default is None:
            default = []

        for file in self._get_input_vfs(
            process_archives=process_archives, force_download=force_download
        ).files:
            if isinstance(file, vfs.FileInContainer):
                yield file.open_concrete(delete=False).name
            elif isinstance(file, vfs.FileOnDisk):
                yield file.path

        return default

    def path(
        self,
        default: Optional[str] = None,
        process_archives: bool = True,
        force_download: bool = False,
    ) -> Optional[str]:
        """Get path to a file for a given input name.

        Returns a file system path for an input.
        If the input contains multiple files, only the first one is returned.
        If the input is not found in the cache, it is downloaded automatically.

        See stream() or paths() for an alternative.

        :param process_archives: When facing an archive file, is it unpacked or returned as is
        :param default: Default fallback path.
        :param force_download: Force re-download of file(s) even when they are cached already.
        :return: File system path to a file for this input.
        """
        input_paths = self.paths(
            process_archives=process_archives, force_download=force_download
        )
        return next(input_paths, default)

    def streams(
        self, process_archives: bool = True, force_download: bool = False
    ) -> Iterator[IO]:
        """Get file streams to all files for a given input name.

        Returns an Iterable for all the file IO streams for an input.

        If the file(s) are not found in the cache, they are downloaded automatically.

        See stream() or paths() for an alternative.

        :param process_archives: When facing an archive file, is it unpacked or returned as is
        :param force_download: Force re-download of file(s) even when they are cached already.
        :return: Iterable for all the IO streams of files for this input.
        """
        for file in self._get_input_vfs(
            process_archives=process_archives, force_download=force_download
        ).files:
            yield file.open()

    def stream(
        self, process_archives: bool = True, force_download: bool = False
    ) -> Optional[IO]:
        """Get a stream for to a file for a given input name.

        Returns an IO stream to a file for this input.
        If the input contains multiple files, only the stream to the first one is returned.
        If the file(s) are not found in the cache, they are downloaded automatically.

        See path() or streams() for an alternative.

        :param process_archives: When facing an archive file, is it unpacked or returned as is
        :param force_download: Force re-download of file(s) even when they are cached already.
        :return: IO stream to a file for this input.
        """

        streams = self.streams(
            process_archives=process_archives, force_download=force_download
        )
        return next(streams, None)

    def _get_input_vfs(
        self, process_archives: bool = True, force_download: bool = False
    ) -> vfs.VFS:
        v = vfs.VFS()
        ii = load_input_info(
            self.name,
            download=DownloadType.ALWAYS if force_download else DownloadType.OPTIONAL,
        )
        if ii:
            for file_info in ii.files:
                vfs.add_disk_file(
                    v,
                    name=file_info.name,
                    path=file_info.path,
                    process_archives=process_archives,
                )
        return v


inputs = Input



import valohai

params = {
    "seq_length": 14,
    "num_epochs": 200,
}

def prepare(a, b):
    print(f"this is fake method {a} {b}")
    
valohai.prepare(step="mystep", default_parameters=params, image="valohai/keras")



import os

import valohai

params = {
    "param1": True,
    "param2": "asdf",
    "param3": 123,
    "param4": 0.0001,
}

inputs = {"input1": "asdf", "input2": ["yolol", "yalala"]}


def prepare(a, b):
    print(f"this is fake method {a} {b}")


valohai.prepare(step="foobar1", default_parameters=params, default_inputs=inputs)



import valohai


def test_get_parameters(use_test_config_dir):
    assert valohai.parameters("foobar").value == 123
    assert valohai.parameters("foobar", 234).value == 123
    assert valohai.parameters("test").value == "teststr"
    assert valohai.parameters("test", "unused_default").value == "teststr"
    assert valohai.parameters("missing_parameter", "hello").value == "hello"
    assert not valohai.parameters("nonono").value



import contextlib
import io
import os
import shutil
import tarfile
import zipfile
from mimetypes import guess_type
from typing import IO, Union

# A short and likely incomplete list of file extensions that are likely not to
# compress well or at all.
INCOMPRESSIBLE_SUFFIXES = {
    "avi",
    "bz2",
    "flac",
    "gif",
    "gz",
    "jpeg",
    "jpg",
    "mkv",
    "mp3",
    "mp4",
    "mpg",
    "npz",
    "ogg",
    "png",
    "rar",
    "tgz",
    "xz",
    "zip",
}


def guess_compressible(name: str) -> bool:
    """
    Guess whether a file with the given name is compressible.
    """
    suffix = os.path.splitext(name)[-1].lower()

    if suffix in INCOMPRESSIBLE_SUFFIXES:
        return False

    type, encoding = guess_type(name, strict=False)
    if encoding:  # Likely a compression
        return False

    if type and "compressed" in type:  # The mime type already says it's compressed
        return False

    return True  # Okay, give it a shot!


class BaseArchive:
    """
    Base class for writable archives with an unified signature for adding files.
    """

    def put(self, archive_name, source: Union[str, IO]):
        raise NotImplementedError("...")


class ZipArchive(BaseArchive, zipfile.ZipFile):
    def __init__(self, file, mode="r", *, compresslevel=1):
        # Only Python 3.7+ has the compresslevel kwarg here
        super().__init__(file, mode, compression=zipfile.ZIP_STORED)
        self.compresslevel = compresslevel

    def writestream(self, arcname, data, compress_type, compresslevel):
        # Like `writestr`, but also supports a stream (and doesn't support directories).
        zinfo = zipfile.ZipInfo(filename=arcname)
        zinfo.compress_type = compress_type
        if hasattr(zinfo, "_compresslevel"):  # only has an effect on Py3.7+
            zinfo._compresslevel = compresslevel
        zinfo.external_attr = 0o600 << 16  # ?rw-------
        # this trusts `open` to fixup file_size.
        with self._lock:
            with self.open(zinfo, mode="w") as dest:
                if isinstance(data, str):
                    dest.write(data.encode("utf-8"))
                if isinstance(data, bytes):
                    dest.write(data)
                else:
                    shutil.copyfileobj(data, dest, 524288)
        assert zinfo.file_size

    def put(self, archive_name, source: Union[str, IO]):
        compress_type = (
            zipfile.ZIP_DEFLATED
            if guess_compressible(archive_name)
            else zipfile.ZIP_STORED
        )
        with contextlib.ExitStack() as es:
            # Python 3.6's `zipfile` does not have `.compresslevel`,
            # so let's just always use our `writestream` instead...
            if isinstance(source, str):
                source = open(source, "rb")
                es.enter_context(source)
            self.writestream(
                arcname=archive_name,
                data=source,
                compress_type=compress_type,
                compresslevel=self.compresslevel,
            )


class TarArchive(BaseArchive, tarfile.TarFile):
    def put(self, archive_name, source: Union[str, IO]):
        with contextlib.ExitStack() as es:
            if isinstance(source, str):
                size = os.stat(source).st_size
                stream = open(source, "rb")
                es.callback(stream.close)
            else:
                # for TAR files we need to know the size of the data beforehand :(
                stream = io.BytesIO(source.read())
                size = len(stream.getbuffer())
            tarinfo = self.tarinfo()
            tarinfo.name = archive_name
            tarinfo.size = size
            self.addfile(tarinfo, stream)


def open_archive(path: str):
    if path.endswith(".zip"):
        return ZipArchive(path, "w")
    elif path.endswith(".tar"):
        return TarArchive.open(path, "w")
    elif path.endswith(".tgz") or path.endswith(".tar.gz"):
        return TarArchive.open(path, "w:gz")

    raise ValueError(f"Unrecognized compression format for {path}")



import json
import os
import pathlib
import zipfile


class ValohaiTestEnvironment:
    def __init__(self, root_dir: str):
        self.root_path = pathlib.Path(root_dir)
        self.config_path = self.root_path / "config"
        self.inputs_path = self.root_path / "inputs"
        self.outputs_path = self.root_path / "outputs"

    def build(self):
        for path in (
            self.root_path,
            self.config_path,
            self.inputs_path,
            self.outputs_path,
        ):
            path.mkdir(exist_ok=True, parents=True)
        (self.config_path / "parameters.json").write_text(
            json.dumps(self.get_parameters())
        )
        (self.config_path / "inputs.json").write_text(json.dumps(self.get_inputs()))

        zip_dir = os.path.join(self.inputs_path, "input_with_archive")
        os.makedirs(zip_dir)
        zip_path = os.path.join(zip_dir, "archive.zip")

        with zipfile.ZipFile(zip_path, "w") as zf:
            zf.writestr("1hello.txt", b"Hernekeitto")
            zf.writestr("2world.txt", b"Viina")

    def get_inputs(self):
        return {
            "single_image": {
                "files": [
                    {
                        "checksums": {
                            "md5": "c8e7e1fc344be6982710f54d47191ef6",
                            "sha1": "628a43d996686e654934e27c99e51afe432fc164",
                            "sha256": "8c712905b799905357b8202d0cb7a244cefeeccf7aa5eb79896645ac50158ffa",
                        },
                        "name": "Example.jpg",
                        "path": f"{self.inputs_path}/single_image/Example.jpg",
                        "size": 27661,
                        "uri": "https://upload.wikimedia.org/wikipedia/en/a/a9/Example.jpg",
                    }
                ]
            },
            "images_in_subdirs": {
                "files": [
                    {
                        "name": "label1/Example.jpg",
                        "path": "%s/images_in_subdirs/label1/Example.jpg"
                        % self.inputs_path,
                        "size": 27661,
                        "uri": "https://upload.wikimedia.org/wikipedia/en/a/a9/Example.jpg",
                    },
                    {
                        "name": "label2/Example.jpg",
                        "path": "%s/images_in_subdirs/label2/Example.jpg"
                        % self.inputs_path,
                        "size": 27661,
                        "uri": "https://upload.wikimedia.org/wikipedia/en/a/a9/Example.jpg",
                    },
                ]
            },
            "input_with_archive": {
                "files": [
                    {
                        "name": "archive.zip",
                        "path": f"{self.inputs_path}/input_with_archive/archive.zip",
                        "checksums": {},
                        "uri": "",
                        "size": 0,
                    }
                ]
            },
        }

    def get_parameters(self):
        return {"foobar": 123, "test": "teststr"}



import pytest

from valohai.internals.parsing import parse

from .utils import get_parsing_tests
from valohai.consts import DEFAULT_DOCKER_IMAGE


def read_test_data():
    """
    Expected files (tests/test_parsing):
        mytest.py -- Python file calling valohai.prepare()
        mytest.inputs.json -- Expected parsed inputs
        mytest.parameters.json -- Expected parsed parameters
        mytest.step.json -- Expected parsed step
    """
    for info in get_parsing_tests():
        yield (
            info["source"],
            info["parameters"],
            info["inputs"],
            info["step"]["name"],
            info["step"]["image"] if 'image' in info["step"] else None,
        )


@pytest.mark.parametrize("source, parameters, inputs, step, image", read_test_data())
def test_parse(source, inputs, parameters, step, image):
    result = parse(source)

    assert result.inputs == inputs
    assert result.parameters == parameters
    assert result.step == step
    assert result.image == image


import os
import tarfile
import zipfile

import pytest

import valohai


@pytest.mark.parametrize("format", ("zip", "tar", "tar.gz"))
@pytest.mark.parametrize("remove_originals", (False, True))
def test_compress(outputs_path, output_files, format, remove_originals):
    filename = f"hello.{format}"
    package_path = valohai.outputs("morjes").compress(
        output_files, filename, remove_originals=remove_originals
    )

    for path in output_files:
        assert os.path.isfile(path) != remove_originals

    # Quick format smoke checks.
    if format == "zip":
        with zipfile.ZipFile(package_path) as zf:
            assert zf.namelist()
    elif "tar" in format:
        with tarfile.open(package_path, "r:*") as tf:
            assert len(list(tf))



import valohai

valohai.prepare(
    step="foobar3",
    default_parameters={
        "param1": True,
        "param2": "asdf",
        "param3": 123,
        "param4": 0.0001,
    },
    default_inputs={"input1": "asdf", "input2": ["yolol", "yalala"]},
)
